package com.redhat.gss.bpms.remote;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.Person;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.api.task.TaskService;
import org.kie.api.task.model.OrganizationalEntity;
import org.kie.api.task.model.Task;
import org.kie.api.task.model.TaskSummary;
import org.kie.remote.client.api.RemoteRuntimeEngineFactory;
import org.junit.Test;

public class RemoteAPIExamples {

	private static final String PROCESS1_ID = "project1.start_and_task_test";
	private static final String PROCESS2_ID = "project1.start_with_pojo_test";
	private static final String URL_STR = "http://localhost:8080/business-central";
	private static final String PASSWORD = "redhat2014!";
	private static final String DEPLOYMENT_ID = "org.kie.example:project1:1.0.0-SNAPSHOT";
	private static final String USER = "jesuino";

	@Test
	public void doTest() throws MalformedURLException {
		URL url = new URL(URL_STR);
		long taskId = startProcessAndHandleTaskViaRestRemoteJavaAPI(url,
				DEPLOYMENT_ID, USER, PASSWORD);
		listPotentialOwners(url, USER, PASSWORD, taskId);
		taskServiceWithoutDeploymentId(url, USER, PASSWORD, taskId);
		startProcessWithParameter(url, DEPLOYMENT_ID, USER, PASSWORD);
	}

	public long startProcessAndHandleTaskViaRestRemoteJavaAPI(URL instanceUrl,
			String deploymentId, String user, String password) {

		// the serverRestUrl should contain a URL similar to
		// "http://localhost:8080/business-central/"
		// Setup the factory class with the necessary information to communicate
		// with the REST services
		RuntimeEngine engine = RemoteRuntimeEngineFactory.newRestBuilder()
				.addUrl(instanceUrl).addUserName(user).addPassword(password)
				.addDeploymentId(deploymentId).build();
		KieSession ksession = engine.getKieSession();
		TaskService taskService = engine.getTaskService();
		// Each operation on a KieSession, TaskService or AuditService (client)
		// instance
		// sends a request for the operation to the server side and waits for
		// the response
		// If something goes wrong on the server side, the client will throw an
		// exception.
		ProcessInstance processInstance = ksession.startProcess(PROCESS1_ID);

		long procId = processInstance.getId();
		String taskUserId = user;
		taskService = engine.getTaskService();
		List<TaskSummary> tasks = taskService.getTasksAssignedAsPotentialOwner(
				user, "en-UK");
		long taskId = -1;

		for (TaskSummary taskSummary : tasks) {
			if (taskSummary.getProcessInstanceId() == procId) {
				taskId = taskSummary.getId();
			}
		}
		if (taskId == -1) {
			throw new IllegalStateException("Unable to find task for " + user
					+ " in process instance " + procId);
		}
		taskService.start(taskId, taskUserId);
		return taskId;
	}

	public void listPotentialOwners(URL instanceUrl, String user,
			String password, Long taskId) {
		RuntimeEngine engine = RemoteRuntimeEngineFactory.newRestBuilder()
				.addUrl(instanceUrl).addUserName(user).addPassword(password)
				.build();
		Task task = engine.getTaskService().getTaskById(taskId);

		List<OrganizationalEntity> org = task.getPeopleAssignments()
				.getPotentialOwners();

		for (OrganizationalEntity ent : org) {
			System.out.println("org: " + ent.getId());
		}
	}

	public void taskServiceWithoutDeploymentId(URL instanceUrl, String user,
			String password, Long taskId) {
		RuntimeEngine engine = RemoteRuntimeEngineFactory.newRestBuilder()
				.addUrl(instanceUrl).addUserName(user).addPassword(password)
				.build();
		// this call doesn't require the deployment id and will return
		// successfully
		engine.getTaskService().complete(taskId, USER, null);
		// this will throw a "MissingRequiredInfoException" because the
		// deployment id is required here
		// engine.getKieSession().startProcess("org.test.process");
	}

	public void startProcessWithParameter(URL instanceUrl, String deploymentId,
			String user, String password) {

		RuntimeEngine engine = RemoteRuntimeEngineFactory.newRestBuilder()
				.addUrl(instanceUrl).addUserName(user).addPassword(password)
				.addExtraJaxbClasses(Person.class)
				.addDeploymentId(deploymentId).build();
		KieSession kSession = engine.getKieSession();

		Map<String, Object> params = new HashMap<>();
		Person person = new Person();
		person.setName("anton");
		params.put("pVar", person);
		ProcessInstance pi = kSession.startProcess(PROCESS2_ID, params);
		System.out.println("Process Started: " + pi.getId());

	}
}